﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace PristaniNaMesici
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        bool connected = false;
        Module module;
        public static bool running = false;
        public static Stopwatch watch = new Stopwatch();

        public MainWindow()
        {
            InitializeComponent();

            new Thread(Update).Start();
            new Thread(UpdateUI).Start();
        }

        private void Update()
        {
            while (true)
            {
                if (running)
                {
                    module.Update(watch.ElapsedMilliseconds / 100000f);
                }

                Thread.Sleep(20);
            }
        }

        void UpdateUI()
        {
            while (true)
            {
                if (running)
                {
                    Application.Current.Dispatcher.Invoke(() => bStart.IsEnabled = false);

                    string[] lines = Regex.Split(Server.GetState(module), "\r\n|\r|\n");

                    int id = 0;
                    double x = 0;
                    double y = 0;
                    double vx = 0;
                    double vy = 0;
                    double targetx = 0;
                    double targety = 0;

                    try
                    {
                        id = int.Parse(lines[0]);
                        x = double.Parse(lines[1].Split(' ')[0], CultureInfo.InvariantCulture);
                        y = double.Parse(lines[1].Split(' ')[1], CultureInfo.InvariantCulture);
                        vx = double.Parse(lines[2].Split(' ')[0], CultureInfo.InvariantCulture);
                        vy = double.Parse(lines[2].Split(' ')[1], CultureInfo.InvariantCulture);
                        targetx = double.Parse(lines[4].Split(' ')[0], CultureInfo.InvariantCulture);
                        targety = double.Parse(lines[4].Split(' ')[1], CultureInfo.InvariantCulture);
                    }
                    catch { }

                    Application.Current.Dispatcher.Invoke(() =>
                    {
                        lCislo.Content = "Identifikační číslo: " + id;
                        lPozice.Content = string.Format("Pozice: {0}, {1}", x, y);
                        lRychlost.Content = string.Format("Rychlost: {0}, {1}", vx, vy);
                        pbPalivo.Value = module.Fuel;
                        lPozicePristani.Content = string.Format("Pozice přistání: {0}, {1}", targetx, targety);

                    });

                    Thread.Sleep(200);
                }
                else if(connected)
                    Application.Current.Dispatcher.Invoke(() => bStart.IsEnabled = true);
            }
        }

        private void bPrihlaseni_Click(object sender, RoutedEventArgs e)
        {
            lPrihlaseni.Content = "Přihlašuji...";

            new Thread(Connect).Start();
        }

        void Connect()
        {
            string adresa = "";

            Application.Current.Dispatcher.Invoke(() => adresa = tbPrihlaseniAdresa.Text);

            if (Server.Connect("http://" + adresa))
            {
                connected = true;
                ConnectUI();
            }
            else
            {
                connected = false;
                DisconnectUI();
            }
        }

        void ConnectUI()
        {
            Application.Current.Dispatcher.Invoke(() => {
                lPrihlaseni.Content = "Přihlášeno";
            });
        }

        void DisconnectUI()
        {
            Application.Current.Dispatcher.Invoke(() => { 
                lPrihlaseni.Content = "Nepřihlášeno";
            });
        }

        private void bStart_Click(object sender, RoutedEventArgs e)
        {
            canvas.Children.Clear();

            module = Module.FromString(Server.Start());
            canvas.Children.Add(module.Image);

            watch.Start();
            running = true;

            Rectangle r = new Rectangle();
            r.Width = 10;
            r.Height = 10;
            r.HorizontalAlignment = HorizontalAlignment.Center;
            r.VerticalAlignment = VerticalAlignment.Center;
            r.Fill = Brushes.Black;
            canvas.Children.Add(r);
            Canvas.SetLeft(r, module.TargetX);
            Canvas.SetTop(r, module.TargetY);

            //label.Content = new WebClient().DownloadString("http://localhost:8000/stav?id=" + module.ID);
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            Environment.Exit(0);
        }

        static List<Key> keyList = new List<Key>();

        protected override void OnKeyDown(KeyEventArgs e)
        {
            base.OnKeyDown(e);

            if (!keyList.Contains(e.Key))
                keyList.Add(e.Key);
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            base.OnKeyUp(e);

            if (keyList.Contains(e.Key))
                keyList.Remove(e.Key);
        }

        public static bool IsKeyPressed(Key key)
        {
            if (keyList.Contains(key))
                return true;
            else return false;
        }
    }
}
